package com.alentoma.selectabletext;

import android.content.Context;
import android.text.Layout;
import android.widget.TextView;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.WritableMap;

public class Utils {

    /**
     * Convert pixels (px) to density-independent pixels (dp).
     *
     * @param context The context to get the display density.
     * @param pxValue The value in pixels (px) to be converted.
     * @return The equivalent value in density-independent pixels (dp).
     */
    public static int pxToDp(Context context, float pxValue) {
        float density = context.getResources().getDisplayMetrics().density;
        return Math.round(pxValue / density);
    }

    /*
        SelectionMap {
            selection: {
                start: number;
                end: number;
                rect: {
                    x: number;
                    y: number;
                    width: number;
                    height: number;
                };
            };
        }
    */
    public static WritableMap getTextSelectionRect(TextView view, int selectionStart, int selectionEnd) {
        Layout layout = view.getLayout();
        if (layout == null) {
            return null;
        }

        int selectionStartLine = layout.getLineForOffset(selectionStart);
        int selectionEndLine = layout.getLineForOffset(selectionEnd);

        boolean isOnSameLine = selectionStartLine == selectionEndLine;

        int selectionStartLineTop = layout.getLineTop(selectionStartLine);
        int selectionEndLineTop = layout.getLineTop(selectionEndLine);

        int selectionStartLineBottom = layout.getLineBottom(selectionStartLine);
        int selectionEndLineBottom = layout.getLineBottom(selectionEndLine);

        int selectionStartX = isOnSameLine ? (int) layout.getPrimaryHorizontal(selectionStart) : 0;
        int selectionEndX = (int) layout.getPrimaryHorizontal(selectionEnd);

        int selectionStartY = selectionStartLineTop;
        int selectionWidth = Math.abs(isOnSameLine ? selectionEndX - selectionStartX : layout.getWidth());
        int selectionHeight = Math.abs(selectionStartLineTop - selectionEndLineBottom);

        Context context = view.getContext();
        WritableMap rect = Arguments.createMap();

        rect.putInt("x", pxToDp(context, selectionStartX));
        rect.putInt("y", pxToDp(context, selectionStartY));
        rect.putInt("width", pxToDp(context, selectionWidth));
        rect.putInt("height", pxToDp(context, selectionHeight));

        return rect;
    }
}
